#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "symbol.h"
#include "dislib.h"
#include "loader.h"
#include "version.h"

void usage(char *exec_name,int help) {
   fprintf(stderr,"\nUsage : %s [--disassemble-all] filename\n\n",exec_name);
   exit(1);
}

    /* The Main Program */
int main(int argc,char **argv) {

    int pc=0,label_line,i;
    short current_instruction;
    char *input_filename=NULL;
    char code[BUFSIZ];
   
    for(i=0;i<argc;i++) {
	 
       if (argv[i][0]=='-') {
	     /* help */
	  if (argv[i][1]=='h') {
	     usage(argv[0],1);
	  }	  
	     /* version */
	  if (argv[i][1]=='v') {
	     usage(argv[0],0);
	  }	
	     /* -- */
	  if (argv[i][1]=='-') {
	     /* ignore.  We assume --disassemble-all */
	  }
       }
       else {
	  input_filename = strdup(argv[i]);
       }
    }
   
    if (input_filename==NULL) {
       usage(argv[0],1);
    }
   
    open_object(input_filename);

    fprintf(stdout,"# Found blj-RiSC executable type %s\n",
	    object.type==OBJECT_NEW?"VMW":"traditional");
    if (object.type==OBJECT_NEW) {   
       fprintf(stdout,"# Found %d sections\n",object.sections);  
    }
    if (num_symbols>0) {
       fprintf(stdout,"# Found %d symbols\n",num_symbols);  
    }

    fprintf(stdout,"\n");

    while (read_short(&current_instruction)>=0) {
       label_line=find_symbol_by_address(pc,SYMBOL_TEXT);
       if (label_line>=0) {
	  fprintf(stdout,"%s:\n",symbol[label_line].label);
       }
       
       if (object.type==OBJECT_NEW) {
	  if (pc*2>=object.text_size) break;  
       }
       
       decode_assembly(current_instruction,pc,code);       
       fprintf(stdout,"\t%04x : %02x %02x : %s\n",
	       pc,(current_instruction>>8)&0xff,current_instruction&0xff,
	       code);
       pc++;       

    }
    close_object();
   
    if (object.type==OBJECT_NEW) {
       if (object.bss_size>0) {
	
	  fprintf(stdout,"\nBSS:\n");
	  fprintf(stdout,"\t%04x : 00 00\n",object.bss_offset);
	  fprintf(stdout,"\t       ...\n");
	  fprintf(stdout,"\t%04x : 00 00\n",(object.bss_offset+
		                          object.bss_size)-2);
       }
    }
   
    if (object.type==OBJECT_NEW) {
       if (num_symbols>0) {
	  fprintf(stdout,"\nSymbols:\n");
	  dump_symbols();
       }
    }
    return 0;  
}
