#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "loader.h"
#include "symbol.h"
#include "io.h"

struct object_type object;

static int bechar_to_int(unsigned char *buffer) {
   
   return (buffer[0]<<24)+(buffer[1]<<16)+(buffer[2]<<8)+buffer[3];
}

int open_object(char *filename) {
   
   unsigned char buffer[4];
   int i,header_size;
   
   object.file=fopen(filename,"r");
   if (object.file==NULL) {
      fprintf(stderr,"ERROR:  cannot open %s\n",filename);
      exit(1);
   }
   
   fread(buffer,sizeof(char),4,object.file);
   if (bechar_to_int(buffer) == 
      (('V'<<24) + ('M'<<16) + ('W'<<8) + '!') ) { 
      
      object.type=OBJECT_NEW;  
      
      fread(buffer,sizeof(char),4,object.file);
      object.sections=bechar_to_int(buffer);

      if (object.sections>MAX_SECTIONS) {
	 fprintf(stderr,"Too many sections: %d\n",object.sections);
	 exit(-1);
      }
      
//      printf("Found %d sections\n",object.sections);
      header_size=(2*sizeof(int))+ /* VMW and # sections */
	           3*object.sections*sizeof(int); /* 3 ints for each section */
      object.header_size=header_size;
//      printf("HEader size: %d %d %x\n",object.sections,
//	     header_size,header_size);
//      printf("Header size=%x\n",header_size);
      
      for(i=0;i<object.sections;i++) {
	 fread(buffer,sizeof(char),4,object.file);
	 object.section[i].type=bechar_to_int(buffer);
	 fread(buffer,sizeof(char),4,object.file);
	 object.section[i].size=bechar_to_int(buffer);	 
	 fread(buffer,sizeof(char),4,object.file);
	 object.section[i].offset=bechar_to_int(buffer);	 
	 
	 if (object.section[i].type==VMW_BSS) {
	    object.bss_size=object.section[i].size;
	    object.bss_offset=object.section[i].offset;
	 }
	 if (object.section[i].type==VMW_TEXT) {
	    object.text_size=object.section[i].size;
	    object.text_offset=object.section[i].offset;
//	    printf("Found %d (%04x) bytes of text at offset %d\n",
//		   object.text_size,
//		   object.text_size,
//		   object.text_offset);
	 }
	 if (object.section[i].type==VMW_SYMBOL) {
	    long current_pos;
	    int symbol_size,symbol_offset;
	    	    
	    current_pos=ftell(object.file);
	    symbol_size=object.section[i].size;
	    symbol_offset=object.section[i].offset;
//	    printf("Looking for symbols at offset: %04x+%04x=%04x\n",
//		   symbol_offset,header_size,
//		   symbol_offset+header_size);
	    read_symbols_from_disk(object.file,
				   symbol_offset+header_size);
	    fseek(object.file,current_pos,SEEK_SET);
	 }	 
      }
      
   }
   else {
      rewind(object.file);
      object.type=OBJECT_OLD;
   }
   
   return 0;
}

int read_short(short *our_short) {
 
   char buffer[2];
   
   if (object.type==OBJECT_OLD) {
      fscanf(object.file,"%hx",our_short);
   }
   else {
      fread(buffer,sizeof(char),2,object.file);      
      *our_short=(buffer[0]<<8)+(buffer[1]&0xff);
   }
   if (feof(object.file)) return -1;
   return 0;
}

int close_object() {
   return fclose(object.file);   
}

int write_header(FILE *object_file, 
		 int symbol_size,int symbol_offset, 
		 int text_size,int text_offset,
		 int bss_size,int bss_offset) {
   
      /* write executable magic */
   write_be_int(object_file,('V'<<24)+('M'<<16)+('W'<<8)+'!');
   
      /* write number of sections */
   if (symbol_size>0) {
      write_be_int(object_file,3);
   }
   else {
      write_be_int(object_file,2);      
   }
   
      /* write text section and size */
   write_be_int(object_file,VMW_TEXT);
   write_be_int(object_file,text_size*2);  /* make it bytes */
   write_be_int(object_file,text_offset);

      /* write bss section and size */
   write_be_int(object_file,VMW_BSS);
   write_be_int(object_file,bss_size*2);   /* make it bytes */
   write_be_int(object_file,bss_offset);
   
   if (symbol_size>0) {
      
         /* write symbol section and size */
      write_be_int(object_file,VMW_SYMBOL);
      write_be_int(object_file,symbol_size); /* already bytes */
      write_be_int(object_file,symbol_offset);
   }
 
   return 0;
}
