#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "io.h"
#include "symbol.h"
#include "loader.h"

struct symbol_type symbol[MAX_SYMBOLS];
int num_symbols=0;

/* return the address of a symbol */
/* printing an error if not found if so desired */
short get_symbol_address(char *s,
			 int error_if_missing,char *error,int line,
			 short text_offset, short bss_offset) {

     int i;

//     fprintf(stderr,"Looking for label %s\n",s);
     for(i=0; i<num_symbols; i++) {
	if (symbol[i].label_length == 0) {
	   break;
	}
	if (strcmp(symbol[i].label,s) == 0) {
	   if (symbol[i].type==SYMBOL_BSS) {
	      return symbol[i].address+bss_offset;
	   }
	   else {
	      return symbol[i].address;
	   }
	}
     }
   
     if (error_if_missing) {
	fprintf(stderr,"ERROR line %d: %s no such symbol as %s\n",
	line,error,s);
	exit(1);
     }   
     return -1;
}

int find_symbol_by_address(short address,int type) {
   
     int i;

     for(i=0; i<num_symbols; i++) {
	if (symbol[i].label_length == 0) {
	   return -1;
	}
	if (symbol[i].address==address) {
	   if (symbol[i].type&type) {
	      return i;
	   }
	}
     }
     return -1;
}


void dump_symbols() {

   int i;
   for(i=0;i<num_symbols;i++) {
      printf("%04x\t%d\t%s\n",
	     symbol[i].address,
	     symbol[i].type,
             symbol[i].label);
   }
}

void dump_symbols_to_disk(FILE *out) {

   int i,j;
   
   write_be_int(out,num_symbols);
   
   for(i=0;i<num_symbols;i++) {
      write_be_int(out,symbol[i].address);
      write_be_int(out,symbol[i].size);
      write_be_int(out,symbol[i].type);      
      write_be_int(out,symbol[i].label_length);
      for(j=0;j<symbol[i].label_length;j++) {
	 fputc(symbol[i].label[j],out);
      }
   }
}

void read_symbols_from_disk(FILE *in,long offset) {
 
   int i,j;
   char temp_string[BUFSIZ];
   
   fseek(in,offset,SEEK_SET);
   
   num_symbols=read_be_int(in);
//   printf("Found %d symbols\n",num_symbols);

   for(i=0;i<num_symbols;i++) {
      symbol[i].address=read_be_int(in);
      symbol[i].size=read_be_int(in);
      symbol[i].type=read_be_int(in);
      symbol[i].label_length=read_be_int(in);
      for(j=0;j<symbol[i].label_length;j++) {
	 temp_string[j]=fgetc(in);
      }
      temp_string[j]='\0';
      symbol[i].label=strdup(temp_string);
   }      
//   dump_symbols();
}

int get_symbol_table_size() {

   int i;
   int size=0;

   size+=sizeof(int); /* for the count */
   for(i=0;i<num_symbols;i++) {
      size+=sizeof(struct symbol_type);
      size+=strlen(symbol[i].label);
   }
   return size;
}

void add_symbol(char *sym,short address,int type) {
 
   if (num_symbols>MAX_SYMBOLS) {
      fprintf(stderr,"Out of symbols trying to add: %s\n",sym);
      exit(1);
   }
   
   symbol[num_symbols].label        = strdup(sym);
   symbol[num_symbols].label_length = strlen(sym);
   symbol[num_symbols].address      = address;
   symbol[num_symbols].type         = type;
   num_symbols++;
}
